==> emelfm2-0.1.8.tar.gz: emelfm2-0.1.8/src/utils/e2_combobox.c <==
/* $Id: e2_combobox.c 2063 2006-07-09 23:22:36Z tpgww $
 *
 * Copyright (C) 2004-2005 tooar <tooar@gmx.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/**
 * @file src/utils/e2_combobox.c
 * @brief GtkComboBox utilities
 *
 * This file contains utility functions for the GtkComboBox widget.
 **/

#include "emelfm2.h"
#include "e2_combobox.h"
#include "e2_pane.h"
#include "e2_command_line.h"

static void _e2_combobox_block_changed (GtkWidget *combo)
{
	g_object_set_data (G_OBJECT (combo), "e2_changed_blocked",
		GINT_TO_POINTER (TRUE));
}

static void _e2_combobox_unblock_changed (GtkWidget *combo)
{
	g_object_set_data (G_OBJECT (combo), "e2_changed_blocked",
		GINT_TO_POINTER (FALSE));
}

  /*********************/
 /***** callbacks *****/
/*********************/

/**
 * @brief key press signal callback to facilitate history handling
 *
 * @param entry the combobox entry widget which received the keypress
 * @param event pointer to key event data struct
 * @param data pointerised gboolean, TRUE to cycle to other end of history
 *
 * @return TRUE if pressed key was <Up> or <Down>
 **/
static gboolean _e2_combobox_key_press_cb (GtkWidget *entry,
	GdkEventKey *event, gpointer data)
{
	gboolean cycle = GPOINTER_TO_INT (data);
	GtkComboBox *combo = GTK_COMBO_BOX (entry->parent);
	gboolean set = FALSE;
	gint pos = -1;
	if (event->keyval == GDK_Up)
	{
		pos = gtk_combo_box_get_active (combo);
		if (pos == -1)
			pos = e2_combobox_last_index (combo);
		else if ((pos > 0) || ((pos == 0) && cycle))
			pos--;
		set = TRUE;
	}
	else
		if (event->keyval == GDK_Down)
	{
		pos = gtk_combo_box_get_active (combo);
		if ((pos != -1 || (cycle)) &&
			(pos < e2_combobox_last_index (combo)))
			pos++;
		else
			pos = -1;
		set = TRUE;
	}
	if (set)
	{
		if (pos == -1)
				gtk_entry_set_text (GTK_ENTRY (entry), "");
		else
		{
			e2_combobox_set_active (GTK_WIDGET (combo), pos);
			gchar *text = e2_combobox_get_active (GTK_WIDGET (combo));
			if (text != NULL)
			{
				gtk_entry_set_text (GTK_ENTRY (entry), text);
				g_free (text);
			}
			gtk_editable_set_position (GTK_EDITABLE (entry), -1);
		}
		return TRUE;
	}
	//work around gtk's failure to replace selected text
	else if (!(event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK))
			&& event->keyval < 0xF000)
	{
		gint start, end;
		if (gtk_editable_get_selection_bounds (GTK_EDITABLE (entry), &start, &end))
		{
			gtk_editable_delete_text (GTK_EDITABLE (entry), start, end);
			gtk_editable_set_position (GTK_EDITABLE (entry), start);
		}
	}
	return FALSE;
}
/**
 * @brief combobox "changed" signal callback
 * When a history item is selected, if @a data is non-NULL (TRUE) the combobox
 * entry is focused. If @a data is NULL (FALSE) then an "activated" signal is
 * emitted on the entry.
 * When the combobox entry is simply edited, nothing is done.
 * @param combo the combobox widget whose entry was changed
 * @param data pointerised gboolean, TRUE if @a combo was created with the
 *	E2_COMBOBOX_FOCUS_ON_CHANGE flag set
 * @return
 */
static void _e2_combobox_changed_cb (GtkWidget *combo, gpointer data)
{
	if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (combo),
		"e2_changed_blocked")))
	{
//		printd (DEBUG, "_e2_combobox_changed_cb BLOCKED");
		return;
	}
	//active item will be -1 whenever an item is edited
	if ((GTK_WIDGET_VISIBLE (combo)) &&
		(gtk_combo_box_get_active (GTK_COMBO_BOX (combo)) != -1))
	{
		gboolean focus = GPOINTER_TO_INT (data);
		GtkWidget *entry = GTK_BIN (combo)->child;
		if (focus)
		{
			gtk_widget_grab_focus (entry);
			gtk_editable_set_position (GTK_EDITABLE (entry), -1);
		}
		else
			g_signal_emit_by_name (G_OBJECT (entry), "activate");
	}
}

  /******************/
 /***** public *****/
/******************/

/**
 * @brief determine the index of the last row in the model of @a combo
 *
 * @param combo the entry text combo box to be interrogated
 *
 * @return the index of the last item in the model, -1 if there's nothing
 **/
gint e2_combobox_last_index (GtkComboBox *combo)
{
	GtkTreeModel *model = gtk_combo_box_get_model (combo);
	guint count = gtk_tree_model_iter_n_children (model, NULL);
	return ((gint)count - 1);
}
/**
 * @brief get the string from the last row in the model of @a combo
 *
 * @param combo the entry text combo box to be interrogated
 *
 * @return newly-allocated copy of the string, or NULL if there's nothing
 **/
gchar *e2_combobox_last_text (GtkComboBox *combo)
{
	GtkTreeModel *mdl = gtk_combo_box_get_model (combo);
	guint count = gtk_tree_model_iter_n_children (mdl, NULL);
	gchar *text = NULL;
	GtkTreeIter iter;
	if (count > 0
		&& gtk_tree_model_iter_nth_child (mdl, &iter, NULL, count-1))
			gtk_tree_model_get (mdl, &iter, 0, &text, -1);
	return text;
}
/**
 * @brief default activate callback
 *
 * This function is the default callback for the "activate" signal of the
 * GtkEntry that belongs to a GtkComboBoxEntry.
 *
 * @param entry entry of a GtkComboBoxentry
 * @param data TRUE if double history entries are allowed
 *
 * @return
 **/
void e2_combobox_activated_cb (GtkWidget *entry, gpointer data)
{
	printd (NOTICE, "e2_combobox_activated_cb");
	gchar *text = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
	GtkComboBox *combo = GTK_COMBO_BOX (entry->parent);
	gtk_combo_box_append_text (combo, text);
	if (GPOINTER_TO_INT (data) == FALSE)
	{
		gint count = e2_combobox_last_index (combo);
		e2_combobox_set_active (GTK_WIDGET (combo), count);
		GtkTreeModel *model = gtk_combo_box_get_model (combo);
		GtkTreeIter iter;
		if (gtk_tree_model_get_iter_first (model, &iter))
			while (e2_tree_find_iter_from_str (model, 0, text, &iter, FALSE))
			{
				GtkTreeIter test;
				test = iter;
				if (gtk_tree_model_iter_next (model, &test))
					gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
				else
					break;
			}
	}
	g_free (text);
}
/**
 * @brief activate the last history item of combo box @a combo
 * This also puts the contents of the history line into the combo
 * entry, and moves the cursor to the end
 * @param combo  GtkComboBox to work on
 * @return
 **/
/*void e2_combobox_select_last (GtkWidget *combo)
{
	GtkWidget *entry = GTK_BIN (combo)->child;
//	GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
	gint count = e2_combobox_last_index ((GTK_COMBO_BOX (combo));
	gchar *text = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
	gint pos = gtk_editable_get_position (GTK_EDITABLE (entry));

//	_e2_combobox_block_changed (combo);	//CHECKME
	e2_combobox_set_active (combo, count);
//	_e2_combobox_unblock_changed (combo);	//CHECKME

	gtk_entry_set_text (GTK_ENTRY (entry), text);
	gtk_editable_set_position (GTK_EDITABLE (entry), pos);
	g_free (text);
} */
/**
 * @brief activate history item @a num of combo box @a combo
 *
 * @param combo  GtkComboBox to work on
 * @param num  the integer index of the item to activate
 * @return
 **/
void e2_combobox_set_active (GtkWidget *combo, gint num)
{
	printd (DEBUG, "e2_combobox_set_active: %d", num);
	_e2_combobox_block_changed (combo);
	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), num);
	_e2_combobox_unblock_changed (combo);
}
/**
 * @brief return currently-selected string from history
 *
 * This function gets the active GtkTreeIter from the GtkComboBox and
 * tries to get the history string from the model and returns it.
 *
 * @param combo  GtkComboBox to work on
 *
 * @return string from history or NULL if nothing is selected
 **/
gchar *e2_combobox_get_active (GtkWidget *combo)
{
	GtkTreeIter iter;
	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter))
	{
		GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
		gchar *retval;
		gtk_tree_model_get (model, &iter, 0, &retval, -1);
		return retval;
	} else
		return NULL;
}
/**
 * @brief append a GList to a GtkComboBox's history
 *
 * This function appends a GList of strings (@a list) to the history
 * of a GtkComboBox.
 *
 * @param combo GtkComboBox to work on
 * @param list GList of strings
 *
 * @return
 **/
void e2_combobox_append_history (GtkWidget *combo, GList *list)
{
	GList *tmp;
	for (tmp = list; tmp != NULL; tmp = g_list_next (tmp))
	{
		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), tmp->data);
	}
}
/**
 * @brief append string array to a GtkComboBox's history
 *
 * This function appends an array of strings (@a strv) to the history
 * of a GtkComboBox.
 *
 * @param combo combobox
 * @param strv a null-terminated array of strings
 *
 * @return
 **/
void e2_combobox_append_history_strv (GtkWidget *combo, gchar **strv)
{
	gint i;
	for (i = 0; strv[i] != NULL; i++)
		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), strv[i]);
}
/**
 * @brief create a GtkComboBox
 *
 * This function creates a GtkComboBox, respecting the @a flags, and then
 * connects @a func to the activate signal.
 *
 * @param func activate callback
 * @param data pointer to E2_CommandLineRuntime struct for the command line, or NULL for an ad-hoc combobox
 * @param flags flags to influence the usage of the GtkComboBox
 *
 * @return the GtkComboBox that was created and packed
 **/
GtkWidget *e2_combobox_get (gpointer func, gpointer data, E2_ComboBoxFlags flags)
{
	gchar *name = "list-style";
	if (!(flags & E2_COMBOBOX_MENU_STYLE))
	{
		//FIXME: work around gtk silliness, the 'appears-as-list'
		//is a style property instead of a widget property
		gchar *str = g_strconcat ("style \"", name, "-style\" { GtkComboBox::appears-as-list = 1 } widget \"*.", name, "\" style \"",
			name, "-style\"", NULL);
		gtk_rc_parse_string (str);
		g_free (str);
	}

	GtkWidget *combo;
	if (flags & E2_COMBOBOX_HAS_ENTRY)
		combo = gtk_combo_box_entry_new_text ();
	else
		combo = gtk_combo_box_new_text ();

	if (!(flags & E2_COMBOBOX_MENU_STYLE))
		gtk_widget_set_name (combo, name);

	if (flags & E2_COMBOBOX_HAS_ENTRY)
	{
		g_signal_connect (G_OBJECT (combo), "changed",
			G_CALLBACK (_e2_combobox_changed_cb),
			GINT_TO_POINTER (flags & E2_COMBOBOX_FOCUS_ON_CHANGE));
	//CHECKME "activate" signal does not exist for a GtkComboBox or any ancestor
	//that signal applies to an entry
		if (!(flags & E2_COMBOBOX_NO_AUTO_HISTORY))
			g_signal_connect (G_OBJECT (GTK_BIN (combo)->child), "activate",
				G_CALLBACK (e2_combobox_activated_cb),
				GINT_TO_POINTER (flags & E2_COMBOBOX_ALLOW_DOUBLE));
	//CHECKME "key_press_event" signal is not relevant for a GtkComboBox
	//that signal applies to an entry
		g_signal_connect (G_OBJECT (GTK_BIN (combo)->child), "key_press_event",
			G_CALLBACK (_e2_combobox_key_press_cb),
			GINT_TO_POINTER (flags & E2_COMBOBOX_CYCLE_HISTORY));
	}
	//CHECKME "activate" signal does not exist for a GtkComboBox or any ancestor
	//that signal applies to an entry
	if (func != NULL)
		g_signal_connect (G_OBJECT (GTK_BIN (combo)->child), "activate",
		G_CALLBACK (func), data);
	return combo;
}
/**
 * @brief pack a GtkComboBox into @a box
 *
 * This function creates a GtkComboBox using e2_combobox_get(), and then packs
 * it into @a box and connects @a func to the "activate" signal.
 *
 * @param box     parent box to add the GtkComboBox to
 * @param expand  packing property
 * @param padding packing property
 * @param func    activate callback
 * @param data    callback data
 * @param flags   flags to influence the usage of the GtkComboBox
 *
 * @return        GtkComboBox that was created and packed
 **/
GtkWidget *e2_combobox_add (GtkWidget *box, gboolean expand, guint padding,
	gpointer func, gpointer data, E2_ComboBoxFlags flags)
{
	GtkWidget *combo = e2_combobox_get (func, data, flags);

	//add padding to the left & right side of the combobox
	GtkWidget *pad_box;
	if (GTK_IS_HBOX (box))
		pad_box = gtk_vbox_new (FALSE, padding);
	else
		pad_box = gtk_hbox_new (FALSE, padding);

	//now add the combo to the padding container
	gtk_box_pack_start (GTK_BOX (pad_box), combo, TRUE, TRUE, padding);
	//add padding container to parent container
	gtk_box_pack_start (GTK_BOX (box), pad_box, expand, FALSE, padding);
	gtk_widget_show (combo);
	gtk_widget_show (pad_box);
	return combo;
}

==> emelfm2-0.2.tar.gz: emelfm2-0.2/src/utils/e2_combobox.c <==
/* $Id: e2_combobox.c 2289 2006-08-25 08:33:46Z tpgww $
 *
 * Copyright (C) 2004-2005 tooar <tooar@gmx.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/**
 * @file src/utils/e2_combobox.c
 * @brief GtkComboBox utilities
 *
 * This file contains utility functions for the GtkComboBox widget.
 **/

#include "emelfm2.h"
#include "e2_combobox.h"
#include "e2_pane.h"
#include "e2_command_line.h"

static void _e2_combobox_block_changed (GtkWidget *combo)
{
	g_object_set_data (G_OBJECT (combo), "e2_changed_blocked",
		GINT_TO_POINTER (TRUE));
}

static void _e2_combobox_unblock_changed (GtkWidget *combo)
{
	g_object_set_data (G_OBJECT (combo), "e2_changed_blocked",
		GINT_TO_POINTER (FALSE));
}

  /*********************/
 /***** callbacks *****/
/*********************/

/**
 * @brief key press signal callback to facilitate history handling
 *
 * @param entry the combobox entry widget which received the keypress
 * @param event pointer to key event data struct
 * @param data pointerised gboolean, TRUE to cycle to other end of history
 *
 * @return TRUE if pressed key was <Up> or <Down>
 **/
static gboolean _e2_combobox_key_press_cb (GtkWidget *entry,
	GdkEventKey *event, gpointer data)
{
	gboolean cycle = GPOINTER_TO_INT (data);
	GtkComboBox *combo = GTK_COMBO_BOX (entry->parent);
	gboolean set = FALSE;
	gint pos = -1;
	if (event->keyval == GDK_Up)
	{
		pos = gtk_combo_box_get_active (combo);
		if (pos == -1)
			pos = e2_combobox_last_index (combo);
		else if ((pos > 0) || ((pos == 0) && cycle))
			pos--;
		set = TRUE;
	}
	else
		if (event->keyval == GDK_Down)
	{
		pos = gtk_combo_box_get_active (combo);
		if ((pos != -1 || (cycle)) &&
			(pos < e2_combobox_last_index (combo)))
			pos++;
		else
			pos = -1;
		set = TRUE;
	}
	if (set)
	{
		if (pos == -1)
				gtk_entry_set_text (GTK_ENTRY (entry), "");
		else
		{
			e2_combobox_set_active (GTK_WIDGET (combo), pos);
			gchar *text = e2_combobox_get_active (GTK_WIDGET (combo));
			if (text != NULL)
			{
				gtk_entry_set_text (GTK_ENTRY (entry), text);
				g_free (text);
			}
			gtk_editable_set_position (GTK_EDITABLE (entry), -1);
		}
		return TRUE;
	}
	//work around gtk's failure to replace selected text
	else if (!(event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK))
			&& event->keyval < 0xF000)
	{
		gint start, end;
		if (gtk_editable_get_selection_bounds (GTK_EDITABLE (entry), &start, &end))
		{
			gtk_editable_delete_text (GTK_EDITABLE (entry), start, end);
			gtk_editable_set_position (GTK_EDITABLE (entry), start);
		}
	}
	return FALSE;
}
/**
 * @brief combobox "changed" signal callback
 * When a history item is selected, if @a data is non-NULL (TRUE) the combobox
 * entry is focused. If @a data is NULL (FALSE) then an "activated" signal is
 * emitted on the entry.
 * When the combobox entry is simply edited, nothing is done.
 * @param combo the combobox widget whose entry was changed
 * @param data pointerised gboolean, TRUE if @a combo was created with the
 *	E2_COMBOBOX_FOCUS_ON_CHANGE flag set
 * @return
 */
static void _e2_combobox_changed_cb (GtkWidget *combo, gpointer data)
{
	if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (combo),
		"e2_changed_blocked")))
	{
//		printd (DEBUG, "_e2_combobox_changed_cb BLOCKED");
		return;
	}
	//active item will be -1 whenever an item is edited
	if ((GTK_WIDGET_VISIBLE (combo)) &&
		(gtk_combo_box_get_active (GTK_COMBO_BOX (combo)) != -1))
	{
		gboolean focus = GPOINTER_TO_INT (data);
		GtkWidget *entry = GTK_BIN (combo)->child;
		if (focus)
		{
			gtk_widget_grab_focus (entry);
			gtk_editable_set_position (GTK_EDITABLE (entry), -1);
		}
		else
			g_signal_emit_by_name (G_OBJECT (entry), "activate");
	}
}

  /******************/
 /***** public *****/
/******************/

/**
 * @brief determine the index of the last row in the model of @a combo
 *
 * @param combo the entry text combo box to be interrogated
 *
 * @return the index of the last item in the model, -1 if there's nothing
 **/
gint e2_combobox_last_index (GtkComboBox *combo)
{
	GtkTreeModel *model = gtk_combo_box_get_model (combo);
	guint count = gtk_tree_model_iter_n_children (model, NULL);
	return ((gint)count - 1);
}
/**
 * @brief get the string from the last row in the model of @a combo
 *
 * @param combo the entry text combo box to be interrogated
 *
 * @return newly-allocated copy of the string, or NULL if there's nothing
 **/
gchar *e2_combobox_last_text (GtkComboBox *combo)
{
	GtkTreeModel *mdl = gtk_combo_box_get_model (combo);
	guint count = gtk_tree_model_iter_n_children (mdl, NULL);
	gchar *text = NULL;
	GtkTreeIter iter;
	if (count > 0
		&& gtk_tree_model_iter_nth_child (mdl, &iter, NULL, count-1))
			gtk_tree_model_get (mdl, &iter, 0, &text, -1);
	return text;
}
/**
 * @brief default activate callback
 *
 * This function is the default callback for the "activate" signal of the
 * GtkEntry that belongs to a GtkComboBoxEntry.
 *
 * @param entry entry of a GtkComboBoxentry
 * @param data TRUE if double history entries are allowed
 *
 * @return
 **/
void e2_combobox_activated_cb (GtkWidget *entry, gpointer data)
{
	printd (NOTICE, "e2_combobox_activated_cb");
	gchar *text = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
	GtkComboBox *combo = GTK_COMBO_BOX (entry->parent);
	gtk_combo_box_append_text (combo, text);
	if (GPOINTER_TO_INT (data) == FALSE)
	{
		gint count = e2_combobox_last_index (combo);
		e2_combobox_set_active (GTK_WIDGET (combo), count);
		GtkTreeModel *model = gtk_combo_box_get_model (combo);
		GtkTreeIter iter;
		if (gtk_tree_model_get_iter_first (model, &iter))
			while (e2_tree_find_iter_from_str (model, 0, text, &iter, FALSE))
			{
				GtkTreeIter test;
				test = iter;
				if (gtk_tree_model_iter_next (model, &test))
					gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
				else
					break;
			}
	}
	g_free (text);
}
/**
 * @brief activate the last history item of combo box @a combo
 * This also puts the contents of the history line into the combo
 * entry, and moves the cursor to the end
 * @param combo  GtkComboBox to work on
 * @return
 **/
/*void e2_combobox_select_last (GtkWidget *combo)
{
	GtkWidget *entry = GTK_BIN (combo)->child;
//	GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
	gint count = e2_combobox_last_index ((GTK_COMBO_BOX (combo));
	gchar *text = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
	gint pos = gtk_editable_get_position (GTK_EDITABLE (entry));

//	_e2_combobox_block_changed (combo);	//CHECKME
	e2_combobox_set_active (combo, count);
//	_e2_combobox_unblock_changed (combo);	//CHECKME

	gtk_entry_set_text (GTK_ENTRY (entry), text);
	gtk_editable_set_position (GTK_EDITABLE (entry), pos);
	g_free (text);
} */
/**
 * @brief activate history item @a num of combo box @a combo
 *
 * @param combo  GtkComboBox to work on
 * @param num  the integer index of the item to activate
 * @return
 **/
void e2_combobox_set_active (GtkWidget *combo, gint num)
{
	printd (DEBUG, "e2_combobox_set_active: %d", num);
	_e2_combobox_block_changed (combo);
	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), num);
	_e2_combobox_unblock_changed (combo);
}
/**
 * @brief return currently-selected string from history
 *
 * This function gets the active GtkTreeIter from the GtkComboBox and
 * tries to get the history string from the model and returns it.
 *
 * @param combo  GtkComboBox to work on
 *
 * @return string from history or NULL if nothing is selected
 **/
gchar *e2_combobox_get_active (GtkWidget *combo)
{
	GtkTreeIter iter;
	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter))
	{
		GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
		gchar *retval;
		gtk_tree_model_get (model, &iter, 0, &retval, -1);
		return retval;
	} else
		return NULL;
}
/**
 * @brief append a GList to a GtkComboBox's history
 *
 * This function appends a GList of strings (@a list) to the history
 * of a GtkComboBox.
 *
 * @param combo GtkComboBox to work on
 * @param list GList of strings
 *
 * @return
 **/
void e2_combobox_append_history (GtkWidget *combo, GList *list)
{
	GList *tmp;
	for (tmp = list; tmp != NULL; tmp = g_list_next (tmp))
	{
		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), tmp->data);
	}
}
/**
 * @brief append string array to a GtkComboBox's history
 *
 * This function appends an array of strings (@a strv) to the history
 * of a GtkComboBox.
 *
 * @param combo combobox
 * @param strv a null-terminated array of strings
 *
 * @return
 **/
void e2_combobox_append_history_strv (GtkWidget *combo, gchar **strv)
{
	gint i;
	for (i = 0; strv[i] != NULL; i++)
		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), strv[i]);
}
/**
 * @brief create a GtkComboBox
 *
 * This function creates a GtkComboBox, respecting the @a flags, and then
 * connects @a func to the activate signal.
 *
 * @param func activate callback
 * @param data pointer to E2_CommandLineRuntime struct for the command line, or NULL for an ad-hoc combobox
 * @param flags flags to influence the usage of the GtkComboBox
 *
 * @return the GtkComboBox that was created and packed
 **/
GtkWidget *e2_combobox_get (gpointer func, gpointer data, E2_ComboBoxFlags flags)
{
	gchar *name = "list-style";
	if (!(flags & E2_COMBOBOX_MENU_STYLE))
	{
		//FIXME: work around gtk silliness, the 'appears-as-list'
		//is a style property instead of a widget property
		gchar *str = g_strconcat ("style \"", name, "-style\" { GtkComboBox::appears-as-list = 1 } widget \"*.", name, "\" style \"",
			name, "-style\"", NULL);
		gtk_rc_parse_string (str);
		g_free (str);
	}

	GtkWidget *combo;
	if (flags & E2_COMBOBOX_HAS_ENTRY)
		combo = gtk_combo_box_entry_new_text ();
	else
		combo = gtk_combo_box_new_text ();

	if (!(flags & E2_COMBOBOX_MENU_STYLE))
		gtk_widget_set_name (combo, name);

	if (flags & E2_COMBOBOX_HAS_ENTRY)
	{
		g_signal_connect (G_OBJECT (combo), "changed",
			G_CALLBACK (_e2_combobox_changed_cb),
			GINT_TO_POINTER (flags & E2_COMBOBOX_FOCUS_ON_CHANGE));
	//CHECKME "activate" signal does not exist for a GtkComboBox or any ancestor
	//that signal applies to an entry
		if (!(flags & E2_COMBOBOX_NO_AUTO_HISTORY))
			g_signal_connect (G_OBJECT (GTK_BIN (combo)->child), "activate",
				G_CALLBACK (e2_combobox_activated_cb),
				GINT_TO_POINTER (flags & E2_COMBOBOX_ALLOW_DOUBLE));
	//CHECKME "key-press-event" signal is not relevant for a GtkComboBox
	//that signal applies to an entry
		g_signal_connect (G_OBJECT (GTK_BIN (combo)->child), "key-press-event",
			G_CALLBACK (_e2_combobox_key_press_cb),
			GINT_TO_POINTER (flags & E2_COMBOBOX_CYCLE_HISTORY));
	}
	//CHECKME "activate" signal does not exist for a GtkComboBox or any ancestor
	//that signal applies to an entry
	if (func != NULL)
		g_signal_connect (G_OBJECT (GTK_BIN (combo)->child), "activate",
		G_CALLBACK (func), data);
	return combo;
}
/**
 * @brief pack a GtkComboBox into @a box
 *
 * This function creates a GtkComboBox using e2_combobox_get(), and then packs
 * it into @a box and connects @a func to the "activate" signal.
 *
 * @param box     parent box to add the GtkComboBox to
 * @param expand  packing property
 * @param padding packing property
 * @param func    activate callback
 * @param data    callback data
 * @param flags   flags to influence the usage of the GtkComboBox
 *
 * @return        GtkComboBox that was created and packed
 **/
GtkWidget *e2_combobox_add (GtkWidget *box, gboolean expand, guint padding,
	gpointer func, gpointer data, E2_ComboBoxFlags flags)
{
	GtkWidget *combo = e2_combobox_get (func, data, flags);

	//add padding to the left & right side of the combobox
	GtkWidget *pad_box;
	if (GTK_IS_HBOX (box))
		pad_box = gtk_vbox_new (FALSE, padding);
	else
		pad_box = gtk_hbox_new (FALSE, padding);

	//now add the combo to the padding container
	gtk_box_pack_start (GTK_BOX (pad_box), combo, TRUE, TRUE, padding);
	//add padding container to parent container
	gtk_box_pack_start (GTK_BOX (box), pad_box, expand, FALSE, padding);
	gtk_widget_show (combo);
	gtk_widget_show (pad_box);
	return combo;
}
